// (C) 2001-2012 Altera Corporation. All rights reserved.
// Your use of Altera Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Altera Program License Subscription 
// Agreement, Altera MegaCore Function License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Altera and sold by 
// Altera or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2010 Altera Corporation. All rights reserved.
// Your use of Altera Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Altera Program License Subscription 
// Agreement, Altera MegaCore Function License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Altera and sold by 
// Altera or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// /**
//  * This Verilog HDL file is used for simulation in
//  * the chained DMA design example.
//  */
`timescale 1 ps / 1 ps
//-----------------------------------------------------------------------------
// Title         : PCI Express BFM Root Port Driver for the chained DMA
//                 design example
// Project       : PCI Express MegaCore function
//-----------------------------------------------------------------------------
// File          : altpcietb_bfm_driver_downstream_main.v
// Author        : Altera Corporation
//-----------------------------------------------------------------------------
// Description : This module is driver for the Root Port BFM for the chained DMA
//               design example.
//-----------------------------------------------------------------------------
//
// Abreviation:
//     EP      : End Point
//     RC      : Root complex
//     DT      : Descriptor Table
//     MWr     : Memory write
//     MRd     : Memory read
//     CPLD    : Completion with data
//     MSI     : PCIe Message Signaled Interrupt
//     BDT     : Base address of the descriptor header table in RC memory
//     BDT_LSB : Base address of the descriptor header table in RC memory
//     BDT_MSB : Base address of the descriptor header table in RC memory
//     BRC     : [BDT_MSB:BDT_LSB]
//     DW0     : First DWORD of the descriptor table header
//     DW1     : Second DWORD of the descriptor table header
//     DW2     : Third DWORD of the descriptor table header
//     RCLAST  : RC MWr RCLAST in EP memeory to reflects the number
//               of DMA transfers ready to start
//     EPLAST  : EP MWr EPLAST in shared memeory to reflects the number
//               of completed DMA transfers
//
//-----------------------------------------------------------------------------
// Copyright  2010 Altera Corporation. All rights reserved.  Altera products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//
// This reference design file, and your use thereof, is subject to and governed by
// the terms and conditions of the applicable Altera Reference Design License Agreement.
// By using this reference design file, you indicate your acceptance of such terms and
// conditions between you and Altera Corporation.  In the event that you do not agree with
// such terms and conditions, you may not use the reference design file. Please promptly
// destroy any copies you have made.
//
// This reference design file being provided on an "as-is" basis and as an accommodation
// and therefore all warranties, representations or guarantees of any kind
// (whether express, implied or statutory) including, without limitation, warranties of
// merchantability, non-infringement, or fitness for a particular purpose, are
// specifically disclaimed.  By making this reference design file available, Altera
// expressly does not recommend, suggest or require that this reference design file be
// used in combination with any other product not provided by Altera.
//-----------------------------------------------------------------------------



`define STR_SEP "---------"

module altpcietb_bfm_driver_downstream_main (input clk_in,
                             input INTA,
                             input INTB,
                             input INTC,
                             input INTD,
                             input rstn,
                             output dummy_out);

   // TEST_LEVEL is a parameter passed in from the top level test bench that
   // could control the amount of testing done. It is not currently used.

   // Global parameter
   parameter  TEST_LEVEL            = 1;
   parameter  TL_BFM_MODE           = 1'b0;      // 0 means full stack RP BFM mode, 1 means TL-only RP BFM (remove CFG accesses to RP internal cfg space)
   parameter  TL_BFM_RP_CAP_REG     = 32'h42;    // In TL BFM mode, pass PCIE Capabilities reg thru parameter (- there is no RP config space).
                                                 // {specify:  port type, cap version}
   parameter  TL_BFM_RP_DEV_CAP_REG = 32'h05;    // In TL BFM mode, pass Device Capabilities reg thru parameter (- there is no RP config space).. 
                                                 // {specify:  maxpayld size}  
   localparam DISPLAY_ALL           = 1;
   localparam SCR_MEM               = 2048;  // Share memory base address used by DMA
   localparam SCR_MEMSLAVE          = 64;    // Share memory base address used by RC Slave module
   localparam SCR_MEM_DOWNSTREAM_WR = SCR_MEMSLAVE;
   localparam SCR_MEM_DOWNSTREAM_RD = SCR_MEMSLAVE+2048;
   localparam RCSLAVE_MAXLEN = 10;  // maximum number of read/write
	

	reg [255:0] TEST_NAME;
	reg [255:0] TESTING;
	reg [2:0] compl_status;


   `include "altpcietb_bfm_constants.v"
   `include "altpcietb_bfm_log.v"
   `include "altpcietb_bfm_shmem.v"
   `include "altpcietb_bfm_rdwr.v"
   `include "altpcietb_bfm_configure.v"
   `include "altpcietb_bfm_driver_downstream.v"
	
   // The clk_in and rstn signals are provided for possible use in controlling
   // the transactions issued, they are not currently used.
	
	///////////////////////////////////////////////////////////////////////////////
//
//
// Main Program
//
// Start of the test bench driver altpcietb_bfm_driver
//
   reg activity_toggle;
   reg timer_toggle ;
   time time_stamp ;
   localparam TIMEOUT = 2000000000;

   initial
     begin
        time_stamp = $time ;
        activity_toggle = 1'b0;
        timer_toggle    = 1'b0;
   end

   // behavioral
   always
   begin : main
	
		
      // If you want to relocate the bar_table, modify the BAR_TABLE_POINTER in altpcietb_bfm_shmem.
      // Directly modifying the bar_table at this location may disable overwrite protection for the bar_table
      // If the bar_table is overwritten incorrectly, this will break the testbench functionality.
      parameter bar_table = BAR_TABLE_POINTER; // Default BAR_TABLE_SIZE is 64 bytes
      integer tgt_bar;
      integer dma_bar, rc_slave_bar;
      reg     addr_map_4GB_limit;
      reg     ret_nill ;
      reg [15:0] msi_control_register;
		
		reg [15:0] test_data;
	   reg [31:0] test_data1;
		reg [31:0] link_status_contents_temp;
		reg [15:0] link_status_contents;
		reg [31:0] link_control_contents_temp;
		reg [15:0] link_control_contents;

      // This constant defines where we save the sizes and programmed addresses
      // of the Endpoint Device Under Test BARs
      // reference design.
		
      $value$plusargs("TEST_NAME=%s", TESTING);
		
	   if(TESTING == "FULL")
		
		begin 
      // Setup the Root Port and Endpoint Configuration Spaces
      addr_map_4GB_limit = 1'b0;
      ret_nill = ebfm_display_verb(EBFM_MSG_WARNING,
           "----> Starting ebfm_cfg_rp_ep task 0 FULL");
			  
      ebfm_cfg_rp_ep(
                     bar_table,         // BAR Size/Address info for Endpoint
                     1,                 // Bus Number for Endpoint Under Test
                     1,                 // Device Number for Endpoint Under Test
                     512,               // Maximum Read Request Size for Root Port
                     1,                 // Display EP Config Space after setup
                     addr_map_4GB_limit, // Limit the BAR assignments to 4GB address map
							0
                     );

      activity_toggle <= ~activity_toggle ;

      ///////////////////////////////////////////
      // Test the chained DMA example design

      find_mem_bar(bar_table, 6'b111111, 8, rc_slave_bar);
			
      downstream_loop(
         bar_table,               // Pointer to the BAR sizing and
         rc_slave_bar,            // Pointer to the BAR sizing and
         RCSLAVE_MAXLEN,          // Number of Write/read iteration
         4,                       // downstream wr/rd length in byte
         0,                       // Downstream EP memory address in byte
                                  // (need to be qword aligned)
         64'hBABA_0000_BEBE_0000);// Starting write data value
			
      ret_nill = ebfm_log_stop_sim(1);
		
		end
		
		
		
		else if(TESTING == "SIMPLE")
		
		begin
		
		// Setup the Root Port and Endpoint Configuration Spaces
      addr_map_4GB_limit = 1'b0;
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO,
				  "----> Starting ebfm_cfg_rp_ep (verbose) SIMPLE");
				  

			ebfm_cfg_rp_ep(bar_table,      
								1,              
								1,               
								512,             
								1,                 
								addr_map_4GB_limit,
								1
								);	

         ret_nill = ebfm_display(EBFM_MSG_INFO, "BEGINNING CONFIGURATION REGISTER READS/WRITES");	
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");	
			
			/* LINK CAPABILITIES REGISTER */			
			ebfm_cfgrd_wait(   1, 				
									 1, 				 
									 0, 					
									 12'h08C, 				
									 4, 					
									 32'h0000_0000, 	
									 compl_status 		
									);
									
									
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");							 
			ret_nill = ebfm_display(EBFM_MSG_INFO, {"EP PCI Express Link Capabilities Register contents: (", himage8(shmem_read(32'h0000_0000, 4)), "):"});
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");	
			
			
			/* LINK STATUS REGISTER */
			ebfm_cfgrd_wait(   1, 				
									 1, 				
									 0, 						
									 12'h090, 			
									 4, 					
									 32'h0000_0004, 	
									 compl_status 			
									 );
									 
									 
			link_status_contents_temp = shmem_read(32'h0000_0004,4);
			link_status_contents = link_status_contents_temp[31:16];
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");		
			ret_nill = ebfm_display(EBFM_MSG_INFO, {"EP PCI Express Link Status Register contents: (", himage4(link_status_contents), "):"});
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");	
			
			
			/* LINK CONTROL REGISTER */
			// Initial Read of the Link Control Register
			ebfm_cfgrd_wait(   1, 			
									 1, 		
									 0, 				
									 12'h090, 		
									 4, 					
									 32'h0000_0008, 	
									 compl_status 			
									 );
									 				 
									 
			link_control_contents_temp = shmem_read(32'h0000_0008,4);
			link_control_contents      = link_control_contents_temp[15:0];
			
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");	
			ret_nill = ebfm_display(EBFM_MSG_INFO, {"Original EP PCI Express Link Control Register contents: (", himage4(link_control_contents), "):"});
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");	
			
			
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");	
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "Writing a '1' to the Retain Link bit (bit 5) of the Link Control Register");	
			test_data = shmem_read(32'h0000_0008, 2);
			test_data = test_data | 16'b0000000000100000;
			shmem_write(32'h0000_0008, test_data, 2);
			link_status_contents_temp[15:0] = test_data;
			shmem_write(32'h0000_0008, link_status_contents_temp, 4);

			
			//writing the modified contents of the Link Control Register back to the Link Control Register
			ebfm_cfgwr_imm_wait(0, 			
									  0, 				
									  0, 						
									  144, 		
									  4, 					
									  link_status_contents_temp, 	
									  compl_status 		
									  );	

			
			//read the contents of the Link Control Register to BFM shared memory
			ebfm_cfgrd_wait(   1, 			
									 1, 		
									 0, 				
									 144, 		
									 4, 					
									 32'h0000_0008, 	
									 compl_status 			
									 ); 
									 
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");	
			
			
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "");
         ret_nill = ebfm_display(EBFM_MSG_INFO, "COMPLETED CONFIGURATION REGISTER READS/WRITES");	
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");	
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "");
		

         ret_nill = ebfm_display(EBFM_MSG_INFO, "BEGINNING DOWNSTREAM MEMORY READS/WRITES");	
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");
			
			/* Memory Read/Writes */
			// locate a suitable BAR
			find_mem_bar(bar_table, 6'b111111, 8, rc_slave_bar);
			
		
			shmem_fill(SCR_MEM_DOWNSTREAM_WR,  
						  SHMEM_FILL_QWORD_INC,  
						  4, 							  
						  64'h4444_3333_2222_1111	
						  ); 
						  
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "");		  


			 // Memory write
			 ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "");	
			 ret_nill = ebfm_display_verb(EBFM_MSG_INFO, {"Writing (",himage8(shmem_read(SCR_MEM_DOWNSTREAM_WR,4)),") to BAR #", himage1(rc_slave_bar)," with address (",himage8(bar_table),")"});	


			 ebfm_barwr(bar_table,
						  rc_slave_bar,
						  0,    				  	 
						  SCR_MEM_DOWNSTREAM_WR,
						  4,				 		    
						  0					     
						  );


			// Memory read
			ebfm_barrd_wait(bar_table,
							    rc_slave_bar,
								 0,
								 SCR_MEM_DOWNSTREAM_RD,
								 4,
								 0
								 ); 

			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, {"Read (",himage8(shmem_read(SCR_MEM_DOWNSTREAM_RD,4)),") from BAR #", himage1(rc_slave_bar)," with address (",himage8(bar_table),")"});					 
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "");	
			
         scr_memory_compare(4, SCR_MEM_DOWNSTREAM_WR, SCR_MEM_DOWNSTREAM_RD);
									 
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "");
			ret_nill = ebfm_display(EBFM_MSG_INFO, "COMPLETED DOWNSTREAM MEMORY READS/WRITES");	
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "---------------------------------------------------------");
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "");
			
			ret_nill = ebfm_log_stop_sim(1);
			
		end

		else
		begin
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "");
			ret_nill = ebfm_display(EBFM_MSG_ERROR_FATAL, "Simulation Stopped - did not specify a test level (edit +TEST_NAME value in msim_setup.tcl)");
			ret_nill = ebfm_display_verb(EBFM_MSG_INFO, "");
		end
		
      forever #100000;
   end

   always
     begin
        #(TIMEOUT)
          timer_toggle <= ! timer_toggle ;
     end

   // purpose: this is a watchdog timer, if it sees no activity on the activity
   // toggle signal for 200 us it ends the simulation
   always @(activity_toggle or timer_toggle)
     begin : watchdog
        reg dummy ;

        if ( ($time - time_stamp) >= TIMEOUT)
          begin
             dummy = ebfm_display(EBFM_MSG_ERROR_FATAL, "Simulation stopped due to inactivity!");
          end
        time_stamp <= $time ;
     end

endmodule
